# -*- coding: utf-8 -*-
from main import *
from vec2d import *

class Projectile(Sprite):
    """A projectile object, fired by a tower upon a target."""
    def __init__(self, screen, game, tower, target):
        """Launch another projectile.
            screen:
                The screen on which the projectile is drawn and located.
            game:
                This is the game object that holds information about the game world.
            tower:
                This is the tower object firing the projectile_image
            target:
                This is the target of the projectile.

            projectile_image:
                is found within the tower object (tower.projectile_image).

        """

        Sprite.__init__(self)

        self.screen = screen
        self.game = game
        self.tower = tower
        self.target = target
        self.speed = self.tower.projectile_speed
        self.pos = vec2d((self.tower.pos[0] + (self.tower.width / 2), self.tower.pos[1] + (self.tower.height / 2)))

        self.base_image = self.tower.projectile_image
        self._compute_direction()
        self.image = pygame.transform.rotate(
            self.base_image, -self.direction.angle)

        self.width = self.base_image.get_width()
        self.height = self.base_image.get_height()
        self.rect = Rect(self.pos[0], self.pos[1], self.width, self.height)
        self.radius = 4 #used for circle collision detection


        self.age = 0
        self.duration = 1 * 1000
        self.game.projectiles.add(self)



    def update(self, time_passed):
        if self.target.is_alive():
            ##for creep in self.game.creeps:
            ##Collision = pygame.sprite.collide_circle(creep, self) #Collision detection with the tower's range
            ##if Collision:
            ##creep._decrease_health(self.tower.damage)
            ##self.kill()

            Collision = pygame.sprite.collide_circle(self.target, self) #Collision detection with the tower's range
            if Collision:
                self.target._decrease_health(self.tower.damage, attacker=self)
                self.kill()
            else:
                self._compute_direction()

                self.target_mid = vec2d(
                    (self.target.rect[0] - (self.target.width / 2)),
                    (self.target.rect[1] - (self.target.width / 2)))

                self.mid_point = vec2d(
                    (self.target.rect[0] - (self.target.width / 2)),
                    (self.target.rect[1] - (self.target.width / 2)))

                remaining_distance = abs(self.target_mid - self.mid_point)

                if remaining_distance > self.speed * time_passed:
                    displacement = vec2d(
                        self.direction.x * self.speed * time_passed,
                        self.direction.y * self.speed * time_passed)
                else:
                    displacement = vec2d(
                        self.direction.x * remaining_distance[0],
                        self.direction.y * remaining_distance[1])

                self.pos += displacement
                self.rect = Rect(self.pos[0], self.pos[1], self.width, self.height)

                self.image = pygame.transform.rotate(
                self.base_image, -self.direction.angle)


                self.age += time_passed
                if self.age > self.duration:
                    self.kill()
        else:
            self.kill()

    def draw(self):
        self.screen.blit(self.image, self.rect)

    def _compute_direction(self):
        aim = self.target.rect
        self.direction = vec2d(
            aim[0] - self.pos[0],
            aim[1] - self.pos[1]).normalized()